/*******************************************************************************
* Freescale Semiconductor Inc.
* (c) Copyright 2010 Freescale Semiconductor, Inc.
* ALL RIGHTS RESERVED.
********************************************************************************
Services performed by FREESCALE in this matter are performed AS IS and without
any warranty. CUSTOMER retains the final decision relative to the total design
and functionality of the end product. FREESCALE neither guarantees nor will be
held liable by CUSTOMER for the success of this project.
FREESCALE DISCLAIMS ALL WARRANTIES, EXPRESSED, IMPLIED OR STATUTORY INCLUDING,
BUT NOT LIMITED TO, IMPLIED WARRANTY OF MERCHANTABILITY OR FITNESS FOR
A PARTICULAR PURPOSE ON ANY HARDWARE, SOFTWARE ORE ADVISE SUPPLIED 
TO THE PROJECT BY FREESCALE, AND OR NAY PRODUCT RESULTING FROM FREESCALE 
SERVICES. IN NO EVENT SHALL FREESCALE BE LIABLE FOR INCIDENTAL OR CONSEQUENTIAL 
DAMAGES ARISING OUT OF THIS AGREEMENT.
CUSTOMER agrees to hold FREESCALE harmless against any and all claims demands 
or actions by anyone on account of any damage, or injury, whether commercial,
contractual, or tortuous, rising directly or indirectly as a result 
of the advise or assistance supplied CUSTOMER in connection with product, 
services or goods supplied under this Agreement.
********************************************************************************
* File:             main.c
* Owner:            stnp002
* Version:          1.0
* Date:             Jan-25-2011
* Classification:   General Business Information
* Brief:            
********************************************************************************
* Detailed Description: 
* 
* This example shows usage of CTU and ADC. Also gives an example of eTimer init.
* The CTU is triggered each 1ms by eTimer0 channel2. The CTU sends sequence of
* 3 commands to ADC0, thus ADC0 ch0 to ch2 are sampled. After command sequence is 
* completed, the CTU interrupt is called.
* eTimer0 channels0 and 1 and eTimer1 channel5 is used to generate PWM signals
* (about 3.9Khz), while its duty cycle is changed based on ADC results.  
* As eTimer1 channel5 on PD4 pin is connected to LED1, it is possible to dim the LED1 
* based on voltage on ADC0 ch0 input.    
* 
* ------------------------------------------------------------------------------
* Test HW:  MPC5604PKIT
* Maskset:  0M36W
* Target :  internal_RAM
* Terminal: no
* Fsys:     64 MHz with 40 MHz XOSC reference
* EVB connection: 
* 
* PJ9.1 - A[0] .. eTimer0 channel0 output PWM signal
* PJ9.2 - A[1] .. eTimer0 channel1 output PWM signal
* PJ9.3 - A[2] .. eTimer0 channel2 output signal, used for CTU triggering
* PJ7.8 - B[7] .. ADC0 AN[0] input
* PJ7.9 - B[8] .. ADC0 AN[1] input
* PJ3.2 - C[1] .. ADC0 AN[2] input
* PJ3.15 - C[14] .. CTU0 EXT TRG output signal
* PJ8.1 - D[0] .. GPIO output, used to show CTU interrupt routine execution   
* PJ8.5 - D[4] .. eTimer1 channel 5 output PWM signal, connected to LED1
*
* connect Trimmer J18.2 to PJ7.8 to dimm the LED1
*
********************************************************************************
Revision History:
1.0     Apr-14-2010     stnp002  Initial Version
*******************************************************************************/
#include "MPC5604P_M26V.h"

/*******************************************************************************
* Global variables
*******************************************************************************/
uint32_t ADC_result[3];

/*******************************************************************************
* Constants and macros
*******************************************************************************/

/*******************************************************************************
* Local types
*******************************************************************************/

/*******************************************************************************
* Local function prototypes
*******************************************************************************/
static void InitModesAndClks(void);
static void InitPeriClkGen(void);
static void DisableWatchdog(void);
static void InitHW(void);
static void InitGPIO(void);
static void InitCTU(void);
static void InitADC(void);
static void InitTimer(void);
static void CTU_ADC_ISR(void);


/*******************************************************************************
* Local variables
*******************************************************************************/

/*******************************************************************************
* Local functions
*******************************************************************************/

/*******************************************************************************
Function Name : InitHW
Engineer      : stnp002
Date          : Jan-25-2011
Parameters    : NONE
Modifies      : NONE
Returns       : NONE
Notes         : init hardware for the purposes of this example code
Issues        : NONE
*******************************************************************************/
static void InitHW(void)
{
    DisableWatchdog();
    InitModesAndClks();
    InitPeriClkGen();     
}

/*******************************************************************************
Function Name : InitModesAndClks
Engineer      : stnp002
Date          : Jan-25-2011
Parameters    : NONE
Modifies      : NONE
Returns       : NONE
Notes         : Enter RUN0 with Fsys 64 MHz PLL with 40 MHz crystal reference.
                monitor Fsys on CLKOUT pin = PB6 = PJ7.7
Issues        : NONE
*******************************************************************************/
static void InitModesAndClks(void) 
{
   	ME.MER.R = 0x0000001D;          // Enable DRUN, RUN0, SAFE, RESET modes */
                                  /* Initialize PLL before turning it on: */
	/* Use 2 of the next 4 lines depending on crystal frequency: */
	/*CGM.CMU_0_CSR.R = 0x000000004; */ /* Monitor FXOSC > FIRC/4 (4MHz); no PLL monitor */
	/*CGM.FMPLL[0].CR.R = 0x02400100;*/ /* 8 MHz xtal: Set PLL0 to 64 MHz */   
  	CGM.CMU_0_CSR.R = 0x000000000;  /* Monitor FXOSC > FIRC/1 (16MHz); no PLL monitor*/
  	CGM.FMPLL[0].CR.R = 0x12400100; /* 40 MHz xtal: Set PLL0 to 64 MHz */   
  	ME.RUN[0].R = 0x001F0074;       /* RUN0 cfg: 16MHzIRCON,OSC0ON,PLL0ON,syclk=PLL0 */
  	ME.RUNPC[0].R = 0x000000FE; 	  /* Peri. Cfg. 0 settings: run in all modes */
  	ME.PCTL[4].R  = 0x00; 	      /* MPC56xxB/P/S DSPI0: select ME.RUNPC[0] */
  	ME.PCTL[32].R = 0x00; 	      /* MPC56xxB/P/S ADC 0: select ME.RUNPC[0] */
  	ME.PCTL[33].R = 0x00; 	      /* MPC56xxB/P/S ADC 1: select ME.RUNPC[0] */
  	ME.PCTL[35].R = 0x00; 	      /* MPC56xxB/P/S CTU 0: select ME.RUNPC[0] */
  	ME.PCTL[41].R = 0x00; 	      /* MPC56xxB/P/S FlexPWM: select ME.RUNPC[0] */
  	ME.PCTL[38].R = 0x00; 	      /* MPC56xxB/P/S eTIMER0: select ME.RUNPC[0] */
  	ME.PCTL[39].R = 0x00; 	      /* MPC56xxB/P/S eTIMER1: select ME.RUNPC[0] */	
                                  /* Mode Transition to enter RUN0 mode: */
  	ME.MCTL.R = 0x40005AF0;         /* Enter RUN0 Mode & Key */
  	ME.MCTL.R = 0x4000A50F;         /* Enter RUN0 Mode & Inverted Key */  
  	while (ME.GS.B.S_MTRANS) {}     /* Wait for mode transition to complete */    
                                  /* Note: could wait here using timer and/or I_TC IRQ */
  	while(ME.GS.B.S_CURRENTMODE != 4) {} /* Verify RUN0 is the current mode */
    
    
    SIU.PCR[22].R = 0x0600;    /* enable CLKOUT on PB6 */
    
    /* set CLKOUT divider of 4 */
    CGM.OCDSSC.B.SELDIV = 0b10;  /* output selected Output Clock divided by 4 */
    CGM.OCDSSC.B.SELCTL = 0b0010;  /* fmpll_0 */
    CGM.OCEN.B.EN = 1; 			 /* enable CLKOUT signal */
}

/*******************************************************************************
Function Name : InitPeriClkGen
Engineer      : stnp002
Date          : Jan-25-2011
Parameters    : NONE
Modifies      : NONE
Returns       : NONE
Notes         : Enable auxiliary clock 0
Issues        : NONE
*******************************************************************************/
static void InitPeriClkGen(void) 
{
    CGM.AC0SC.R = 0x04000000;  /* MPC56xxP ADC: Select IPBClk for aux clk 0  */
    CGM.AC0DC.R = 0x80000000;  /* MPC56xxP ADC: Enable aux clk 0 div by 1 */

} 

/*******************************************************************************
Function Name : InitGPIO
Engineer      : stnp002
Date          : Jan-25-2011
Parameters    : NONE
Modifies      : NONE
Returns       : NONE
Notes         : IO configuration
Issues        : NONE
*******************************************************************************/
static void InitGPIO(void)
{
        
    SIU.PCR[0].R = 0x0600;    // A[0] pin as eTimer0 channel 0 output
    SIU.PCR[1].R = 0x0600;    // A[1] pin as eTimer0 channel 1 output  
    SIU.PCR[2].R = 0x0600;    // A[2] pin as eTimer0 channel 2 output
    SIU.PCR[23].R = 0x2400;   // B[7] pin as ADC0 AN[0] input
    SIU.PCR[24].R = 0x2400;   // B[8] pin as ADC0 AN[1] input
    SIU.PCR[33].R = 0x2400;   // C[1] pin as ADC0 AN[2] input
    SIU.PCR[46].R = 0x0A00;   // C[14] pin as CTU0 EXT TRG output
    SIU.PCR[48].R = 0x0200;   // D[0] pin as GPIO output   
    SIU.PCR[52].R = 0x0A00;   // D[4] pin as eTimer1 channel 5 output
    
}

/*******************************************************************************
Function Name : DisableWatchdog
Engineer      : stnp002
Date          : Jan-25-2011
Parameters    : NONE
Modifies      : NONE
Returns       : NONE
Notes         : disable SWT functionality
Issues        : NONE
*******************************************************************************/
static void DisableWatchdog(void) 
{
    SWT.SR.R = 0x0000c520;     /* Write keys to clear soft lock bit */
    SWT.SR.R = 0x0000d928; 
    SWT.CR.R = 0xFF00010A;     /* Clear watchdog enable (WEN) */
}

/*******************************************************************************
Function Name : InitCTU
Engineer      : stnp002
Date          : Jan-04-2011
Parameters    : NONE
Modifies      : NONE
Returns       : NONE
Notes         : CTU initialization
Issues        : NONE
*******************************************************************************/
static void InitCTU(void)
{
	
    CTU_0.TGSISR.R = 0x04000000; // Enable eTimer0.channel2 rising edge trigger; I13_RE

    CTU_0.TCR[0].R = 0x2000;     /* For triggering conversion sequence of ADC0-CH0, ADC0-CH1 and ADC0-CH2
    								determines delay between external trigger and triggering ADC sequence*/

    /* Counter */
    CTU_0.TGSCCR.R              = 0x7d00;
    CTU_0.TGSCRR.R              = 0x0000;
    CTU_0.TGSCR.R               = 0x0000;

    CTU_0.THCR1.R               = 0x00000019; /* Enable Triger 0 and set to ADC */

    /* Set ADC messages */
    CTU_0.CLR[0].R              = 0x4000;  // Command 0 - first command in command sequence, ADC0-CH0 conversion 
    CTU_0.CLR[1].R              = 0x0401;  // Command 1 - ADC0-CH1 conversion
    CTU_0.CLR[2].R              = 0x8802;  // Command 2 - ADC0-CH2 conversion, issue interrupt after conversion 
   
    CTU_0.CTUCR.R               = 0x0003; /* General reload and TGS input selection reload */
}

/*******************************************************************************
Function Name : InitADC
Engineer      : stnp002
Date          : Jan-04-2011
Parameters    : NONE
Modifies      : NONE
Returns       : NONE
Notes         : ADC initialization
Issues        : NONE
*******************************************************************************/
static void InitADC(void)
{

    ADC_0.MCR.R		= 0x80020000;	// enable result overwriting, enable CTU triggered injected conversion
    ADC_0.CTR[0].R  = 0x00008606;   // conversion times for 32MHz ADClock

}

/*******************************************************************************
Function Name : InitTimer
Engineer      : stnp002
Date          : Jan-04-2011
Parameters    : NONE
Modifies      : NONE
Returns       : NONE
Notes         : eTimer initialization
Issues        : NONE
*******************************************************************************/
static void InitTimer(void)
{
    
    /* Set eTimer0 channel2 to produce 1Khz trigger signal for CTU module*/
	
	ETIMER_0.CHANNEL[2].COMP1.R = 32000;	// Count until this value, 64Mhz/32000=2Khz -> toggle every 500us 
	ETIMER_0.CHANNEL[2].LOAD.R = 0x0;
	ETIMER_0.CHANNEL[2].CNTR.R = 0x0;
	ETIMER_0.CHANNEL[2].CCCTRL.R = 0x0200;  // Compare mode 10; COMP1 register is used when the counter is counting up.
	ETIMER_0.CHANNEL[2].CTRL2.R = 0x8003;	// Output enable, Toggle OFLAG output on successful compare 
	ETIMER_0.CHANNEL[2].CTRL.R = 0x3840;	/* Counts only rising edge of the Fsys (64MHz in RUN0), 
											divide by 1, count up until COMP1 then reinit, 
											count repeatedly */
	
	
	/* Set eTimer0 channels 0 and 1 and eTimer1 channel5 for PWM variable frequency mode */ 

	ETIMER_0.CHANNEL[0].COMP1.R = 0;
	ETIMER_0.CHANNEL[0].COMP2.R = 1024;
	ETIMER_0.CHANNEL[0].CMPLD1.R = 0;
	ETIMER_0.CHANNEL[0].CMPLD2.R = 1024;
	ETIMER_0.CHANNEL[0].LOAD.R = 0x0;
	ETIMER_0.CHANNEL[0].CNTR.R = 0x0;
    ETIMER_0.CHANNEL[0].CCCTRL.R = 0x8C00;  /* Load COMP2 with CMPLD2 upon successful compare with the value in COMP1
    										   Load COMP1 with CMPLD1 upon successful compare with the value in COMP2. */
	ETIMER_0.CHANNEL[0].CTRL2.R = 0x8004;   // Output enable, Toggle OFLAG output using alternating compare registers
	ETIMER_0.CHANNEL[0].CTRL.R = 0x3c40;
	
	ETIMER_0.CHANNEL[1].COMP1.R = 0;
	ETIMER_0.CHANNEL[1].COMP2.R = 1024;
	ETIMER_0.CHANNEL[1].CMPLD1.R = 0;
	ETIMER_0.CHANNEL[1].CMPLD2.R = 1024;
	ETIMER_0.CHANNEL[1].LOAD.R = 0x0;
	ETIMER_0.CHANNEL[1].CNTR.R = 0x0;
	ETIMER_0.CHANNEL[1].CCCTRL.R = 0x8C00;
	ETIMER_0.CHANNEL[1].CTRL2.R = 0x8004;
	ETIMER_0.CHANNEL[1].CTRL.R = 0x3c40;
	
	ETIMER_1.CHANNEL[5].COMP1.R = 0;
	ETIMER_1.CHANNEL[5].COMP2.R = 1024;
	ETIMER_1.CHANNEL[5].CMPLD1.R = 0;
	ETIMER_1.CHANNEL[5].CMPLD2.R = 1024;
	ETIMER_1.CHANNEL[5].LOAD.R = 0x0;
	ETIMER_1.CHANNEL[5].CNTR.R = 0x0;
	ETIMER_1.CHANNEL[5].CCCTRL.R = 0x8C00;
	ETIMER_1.CHANNEL[5].CTRL2.R = 0x8004;
	ETIMER_1.CHANNEL[5].CTRL.R = 0x3c40;
	
}
/*******************************************************************************
Function Name : main
Engineer      : stnp002
Date          : Jan-04-2011
Parameters    : NONE
Modifies      : NONE
Returns       : NONE
Notes         : main function
Issues        : NONE
*******************************************************************************/
int main(void) 
{
    volatile int i = 0;

    InitHW();  
    
    InitGPIO();
    InitCTU();
    InitADC();
    InitTimer();
    
    INTC_InstallINTCInterruptHandler(CTU_ADC_ISR, 206,2); /* install an interrupt handler (CTU_ADC_EOC) 
  														  for a given interrupt vector (206) and 
  														  set the interrupt priority (2) */
  
    INTC.CPR.B.PRI = 0;	/* Lower INTC's current priority, by default highest, e.g 15, is used */
  

    /* Loop forever */
    for (;;) 
    {
      i++;
    }
}

/*******************************************************************************
Function Name : CTU_ADC_ISR
Engineer      : stnp002
Date          : Jan-04-2011
Parameters    : NONE
Modifies      : NONE
Returns       : NONE
Notes         : CTU_ADC command interrupt routine
Issues        : NONE
*******************************************************************************/
static void CTU_ADC_ISR(void)
{
    int i = 0;
  
    SIU.GPDO[48].R = 1; // Set D[0] pin
     
    /* Read ADC results from ADC Channel Data Registers */
    
    ADC_result[0] = (ADC_0.CDR[0].R);
    ADC_result[1] = (ADC_0.CDR[1].R);
    ADC_result[2] = (ADC_0.CDR[2].R);
  
    /* Read ADC results from CTU FIFO Right aligned data registers */
 /*   
    ADC_result[0] = (CTU_0.FRA[0].R);
    ADC_result[1] = (CTU_0.FRA[1].R);
    ADC_result[2] = (CTU_0.FRA[2].R);
 */   
    
    /* Update eTimers Compare Load Registers based on ADC values to modify PWM signal duty */
    
    ETIMER_1.CHANNEL[5].CMPLD1.R = (uint16_t)(ADC_result[0] & 0x3ff);
    ETIMER_1.CHANNEL[5].CMPLD2.R = (uint16_t)(1024 - ETIMER_1.CHANNEL[5].CMPLD1.R);
    
    ETIMER_0.CHANNEL[0].CMPLD1.R = (uint16_t)(ADC_result[1] & 0x3ff);
    ETIMER_0.CHANNEL[0].CMPLD2.R = (uint16_t)(1024 - ETIMER_0.CHANNEL[0].CMPLD1.R);
    
    ETIMER_0.CHANNEL[1].CMPLD1.R = (uint16_t)(ADC_result[2] & 0x3ff);
    ETIMER_0.CHANNEL[1].CMPLD2.R = (uint16_t)(1024 - ETIMER_0.CHANNEL[1].CMPLD1.R);
    
    CTU_0.CTUIFR.B.ADC = 0x1;   // ADC ISR flag clear
    
    CTU_0.CTUCR.B.GRE  = 1; // CTU General Reload Enable
    
    for(i=0;i<0xff;i++){};  // just for little delay
    
    SIU.GPDO[48].R = 0;		// Clear D[0] pin
}


